﻿#region Using

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Web.Caching;
using System.Xml;
using System.IO;
using System.Text;
using BlogEngine.Core;

#endregion

public partial class widgets_RandomPosts_widget : WidgetBase
{
	private const int DEFAULT_NUMBER_OF_POSTS = 10;

    static widgets_RandomPosts_widget()
	{
		BlogSettings.Changed += delegate { HttpRuntime.Cache.Remove("widget_randomposts"); };
	}

    public override void LoadWidget()
    {
        StringDictionary settings = GetSettings();
        int numberOfPosts = DEFAULT_NUMBER_OF_POSTS;
        if (settings.ContainsKey("numberofposts"))
            numberOfPosts = int.Parse(settings["numberofposts"]);

        if (HttpRuntime.Cache["widget_randomposts"] == null)
        {

            List<Post> visiblePosts = Post.Posts.FindAll(delegate(Post p)
            {
                return p.IsVisibleToPublic;//仅显示对游客可见文章
            });

            int max = Math.Min(visiblePosts.Count, numberOfPosts);
            List<Post> list = new List<Post>();

            for (int i = max; i > 0; i--)
            {
                Random random = new Random();
                int r = random.Next(0, visiblePosts.Count);//生成随机数

                list.Add(visiblePosts[r]);
                visiblePosts.RemoveAt(r);
            }

            //缓存文章列表，每15分钟更新
            HttpRuntime.Cache.Insert("widget_randomposts", list, null, DateTime.Now.AddMinutes(15), TimeSpan.Zero);
        }

        string content = RenderPosts((List<Post>)HttpRuntime.Cache["widget_randomposts"], settings);

        LiteralControl html = new LiteralControl(content);
        phPosts.Controls.Add(html);
    }

	private string RenderPosts(List<Post> posts, StringDictionary settings)
	{
		if (posts.Count == 0)
		{
			return "<p>" + Resources.labels.none + "</p>";
		}

		StringBuilder sb = new StringBuilder();
		sb.Append("<ul class=\"randomposts\" id=\"randomposts\">");

		foreach (Post post in posts)
		{
			if (!post.IsVisible)
				continue;

            string link = "<li><a href=\"{0}\">{1}</a></li>";
            sb.AppendFormat(link, post.RelativeLink, HttpUtility.HtmlEncode(post.Title));
		}

		sb.Append("</ul>");
		return sb.ToString();
	}

	public override string Name
	{
        get { return "RandomPosts"; }
	}

	public override bool IsEditable
	{
		get { return true; }
	}

}
